package de.unibi.citec.clf.bonsai.actuators;

/*
 * #LICENSE%
 * BonSAI [Core|Base]
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
 * 
 * http://opensource.cit-ec.de/projects/bonsai
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.io.IOException;

import de.unibi.citec.clf.bonsai.core.Actuator;

/**
 * This is a remote control interface to manipulate chains and tunnels in an
 * icewing instance, that runs a XcfChainSwitch plugin.
 * 
 * @author lziegler
 */
public interface IcewingChainSwitchActuator extends Actuator {

    /**
     * Deacitvates a chain.
     * 
     * @param name
     *            The name of the chain to deactive.
     * @throws IOException
     *             Is thrown when a communication error occurs.
     */
    void deactivateChain(String name) throws IOException;

    /**
     * Acitvates a chain.
     * 
     * @param name
     *            The name of the chain to active.
     * @throws IOException
     *             Is thrown when a communication error occurs.
     */
    void activateChain(String name) throws IOException;

    /**
     * Deacitvates a tunnel.
     * 
     * @param name
     *            The name of the tunnel to deactive.
     * @throws IOException
     *             Is thrown when a communication error occurs.
     */
    void deactivateTunnel(String name) throws IOException;

    /**
     * Acitvates a tunnel.
     * 
     * @param name
     *            The name of the tunnel to active.
     * @throws IOException
     *             Is thrown when a communication error occurs.
     */
    void activateTunnel(String name) throws IOException;

}
