package de.unibi.citec.clf.bonsai.actuators;

/*
 * #LICENSE%
 * BonSAI [Core|Base]
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
 * 
 * http://opensource.cit-ec.de/projects/bonsai
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import de.unibi.citec.clf.bonsai.core.Actuator;
import de.unibi.airobots.btl.data.navigation.ROSDynamicConfiguration;
import java.io.IOException;

/**
 * Interface for actuators changing ROS parameters during runtime.
 * 
 * @author pdressel
 */
public interface ROSDynamicReconfigurationActuator<T extends ROSDynamicConfiguration> extends Actuator {
    
    /**
     * Writes the {@link config} to the corresponding  server. Parameters which 
     * differ from the current configuration are going to be overwritten during 
     * runtime.
     * @param config The configuration to apply.
     * @throws IOException 
     */
    public void writeConfig(T config) throws IOException;
    
    /**
     * Reads the current configuration from the corresponding server. (May be 
     * used to initialize a configuration object which is then changed and send
     * back).
     * @return
     *      the configuration read.
     * @throws IOException 
     */
    public T readConfig() throws IOException;    
}
