package de.unibi.citec.clf.bonsai.core;

/*
 * #LICENSE%
 * BonSAI [Core|Base]
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
 * 
 * http://opensource.cit-ec.de/projects/bonsai
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.util.Map;

import de.unibi.citec.clf.bonsai.core.CoreObjectFactory.CoreObjectCreationException;
import de.unibi.citec.clf.bonsai.core.CoreObjectFactory.InitializationException;

/**
 * Interface for factories that create {@link MemorySlot} objects that allow
 * access to distinct information inside a (runtime-) persistent storage unit.
 * 
 * @author lziegler
 * 
 */
public interface WorkingMemory extends ManagedCoreObject {
	
	/**
	 * Configures the working memory with user-supplied options. This is the first
	 * method invoked
	 * 
	 * @param options
	 *            options provided by the user as key-value-pairs.
	 * @throws IllegalArgumentException
	 *             this exception may be thrown to indicate an error in the
	 *             options provided by the user. If this is thrown, the factory
	 *             may be in an undetermined state and should not be used for
	 *             creating Bonsai objects
	 * @throws InitializationException
	 *             a general error initializing this memory occurred. It cannot
	 *             be used after this exception
	 */
	void initialize(Map<String, String> options)
			throws IllegalArgumentException, InitializationException;

	/**
	 * Creates a communication object that allows access to distinct information
	 * inside the storage unit.
	 * 
	 * @param slotName
	 *            A unique identifier for the information that should be stored
	 *            or accessed.
	 * @param dataType
	 *            The type of the data that should be stored or accessed.
	 * @return A {@link MemorySlot} object that grant access to the data
	 *         identified by parameter slotName.
	 * @throws CommunicationException
	 *             When the communication with the storage unit fails.
	 * @throws IllegalArgumentException
	 *             When the configuration of the working memory does not allow
	 *             to create the desired slot.
	 * @throws CoreObjectCreationException
	 *             When an error occurred while creating the requested slot.
	 */
	<T> MemorySlot<T> getSlot(String slotName, Class<T> dataType)
			throws CommunicationException, IllegalArgumentException,
			CoreObjectCreationException;
}
