package de.unibi.citec.clf.bonsai.core.configuration;

/*
 * #LICENSE%
 * BonSAI [Core|Base]
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
 * 
 * http://opensource.cit-ec.de/projects/bonsai
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.util.Map;

/**
 * Configuration of one factory.
 * 
 * @author jwienke
 */
public class ConfiguredCoreObjectFactory {

    private String factoryClassName;
    private Map<String, String> factroyOptions;

    /**
     * Returns the name of the factory class.
     * 
     * @return factory class name
     */
    public String getFactoryClassName() {
        return factoryClassName;
    }

    /**
     * Sets the name of the factory class.
     * 
     * @param factoryClassName
     *            class name
     */
    public void setFactoryClassName(String factoryClassName) {
        this.factoryClassName = factoryClassName;
    }

    /**
     * Returns a map of key-value style options passed to this factory.
     * 
     * @return options
     */
    public Map<String, String> getFactroyOptions() {
        return factroyOptions;
    }

    /**
     * Sets the map of key-value style options passed to a factory.
     * 
     * @param factroyOptions
     *            new options
     */
    public void setFactroyOptions(Map<String, String> factroyOptions) {
        this.factroyOptions = factroyOptions;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof ConfiguredCoreObjectFactory)) {
            return false;
        }
        ConfiguredCoreObjectFactory other = (ConfiguredCoreObjectFactory) obj;
        return other.factoryClassName.equals(factoryClassName)
                && other.factroyOptions.equals(factroyOptions);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int hashCode() {
        // CHECKSTYLE:OFF
        return factoryClassName.hashCode() * 3 + factroyOptions.hashCode() * 7;
        // CHECKSTYLE:ON
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString() {
        return "[" + getClass().getSimpleName() + ": factoryClassName = "
                + factoryClassName + ", factoryOptions: " + factroyOptions
                + "]";
    }

}
