package de.unibi.citec.clf.bonsai.util.reflection;

/*
 * #LICENSE%
 * BonSAI [Core|Base]
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
 * 
 * http://opensource.cit-ec.de/projects/bonsai
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.apache.log4j.Logger;

import sun.misc.Service;

/**
 * An implementation of {@link ServiceDiscovery} that uses the jar file services
 * mechanism to find services.
 * 
 * @see <a
 *      href="http://java.sun.com/javase/6/docs/technotes/guides/jar/jar.html#Service%20Provider">JAR
 *      File Specification - Service Provider</a>
 * @author jwienke
 */
@SuppressWarnings("restriction")
public class JarServiceDiscovery implements ServiceDiscovery {

    private Logger logger = Logger.getLogger(getClass());

    @SuppressWarnings("unchecked")
    @Override
    public <T> Set<Class<? extends T>> discoverServicesByInterface(
            Class<T> iFace) {

        logger.info("Discovering JAR services for interface " + iFace);

        Set<Class<? extends T>> classes = new HashSet<Class<? extends T>>();

        @SuppressWarnings("rawtypes")
		Iterator ps = Service.providers(iFace);
        while (ps.hasNext()) {
            T t = (T) ps.next();
            Class<?> clazz = t.getClass();
            logger.debug("Discovered service class " + clazz);
            classes.add((Class<T>) clazz);
        }

        logger.info("Discovered instances of " + iFace + ": " + classes);

        return classes;

    }

}
