package de.unibi.citec.clf.bonsai.xcf.actuators;

/*
 * #LICENSE%
 * BonSAI [Core|XCF]
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
 * 
 * http://opensource.cit-ec.de/projects/bonsai
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.util.HashMap;
import java.util.Iterator;

import org.apache.log4j.Logger;

import net.sf.xcf.ActiveMemory;
import net.sf.xcf.memory.MemoryException;
import net.sf.xcf.transport.XOPData;
import nu.xom.Attribute;
import nu.xom.Element;
import de.unibi.citec.clf.bonsai.actuators.TaskPatternActuator;
import de.unibi.citec.clf.bonsai.xcf.ActiveMemoryReconnectable;
import de.unibi.airobots.btl.data.taskpattern.TaskOriginData;
import de.unibi.airobots.btl.data.taskpattern.TaskPatternStatusTagData;
import de.unibi.airobots.btl.data.taskpattern.TaskStateData;

/**
 * Implementation of the TaskPatternActuator.
 * 
 * @author arothert, vrichter, bcarlmey
 */
public class XcfTaskPatternActuator implements ActiveMemoryReconnectable,
		TaskPatternActuator {

	private Logger logger = Logger.getLogger(getClass());

	private String memoryName = "";
	private Object memoryLock = new Object();
	private ActiveMemory activeMemory = null;
	private TaskPatternStatusTagData taskStatusTagData = new TaskPatternStatusTagData();

	/**
	 * Xml namespace for active memory documents.
	 */
	public static final String NAMESPACE_MEMORY = "http://purl.org/ActiveMemory";

	/**
	 * Xml namespace for dbxml.
	 */
	public static final String NAMESPACE_DBXML = "http://www.sleepycat.com/2002/dbxml";

	public XcfTaskPatternActuator(String memoryName) {
		this.memoryName = memoryName;
	}

	public void initiate(XOPData taskSpec) {
		// fill the status tag
		taskStatusTagData.fillTaskStatus(TaskOriginData.Submitter,
				TaskStateData.initiated);

		if (taskSpec.getDocument().getRootElement().getChildElements("STATUS")
				.size() < 1) {
			taskSpec.getDocument().getRootElement()
					.appendChild(taskStatusTagData.getTaskStatus());
		}
		// check for correct set attributes within status tag => must be initial
		// and submitter. !
		else if (!taskSpec.getDocument().getRootElement()
				.getChildElements("STATUS").get(0)
				.getAttribute(taskStatusTagData.getOrigin().toString())
				.equals(TaskOriginData.Submitter.toString())
				|| !taskSpec.getDocument().getRootElement()
						.getChildElements("STATUS").get(0)
						.getAttribute(taskStatusTagData.getState().toString())
						.equals(TaskStateData.initiated.toString())) {
			Element elem = taskSpec.getDocument().getRootElement()
					.getChildElements("STATUS").get(0);

			logger.debug("Task initiation with the existing STATUS is not valid! "
					+ elem.toXML()
					+ "\n Status changed to "
					+ taskStatusTagData.getTaskStatus());
			taskSpec.getDocument().getRootElement()
					.replaceChild(elem, taskStatusTagData.getTaskStatus());
		}

		try {
			logger.info("Inserting into AM: " + taskSpec.getDocumentAsText());
			activeMemory.insert(taskSpec);
		} catch (MemoryException e) {
			logger.error(e);
		}

	}

	public void update(XOPData taskSpec, TaskStateData taskState,
			TaskOriginData taskOrigin) {
		// set new Status Tag
		taskStatusTagData.fillTaskStatus(taskOrigin, taskState);

		if (taskSpec.getDocument().getRootElement().getChildElements("STATUS")
				.size() == 1) {

			taskSpec.getDocument()
					.getRootElement()
					.replaceChild(
							taskSpec.getDocument().getRootElement()
									.getChildElements("STATUS").get(0),
							taskStatusTagData.getTaskStatus());
		} else {
			taskSpec.getDocument().getRootElement()
					.appendChild(taskStatusTagData.getTaskStatus());

		}
		try {
			activeMemory.replace(taskSpec);
		} catch (MemoryException e) {
			logger.error(e);
		}

	}

	public void attributeUpdate(XOPData taskSpec, String attribute,
			String value, TaskStateData taskState, TaskOriginData taskOrigin) {
		HashMap<String, String> mapOfAttributes = new HashMap<String, String>();
		mapOfAttributes.put(attribute, value);
		attributeUpdate(taskSpec, mapOfAttributes, taskState, taskOrigin);
	}

	public void attributeUpdate(XOPData taskSpec,
			HashMap<String, String> mapOfAttributes, TaskStateData taskState,
			TaskOriginData taskOrigin) {

		Iterator<String> it = mapOfAttributes.keySet().iterator();
		while (it.hasNext()) {
			String attribute = it.next();
			String value = mapOfAttributes.get(attribute);

			Attribute att = taskSpec.getDocument().getRootElement()
					.getAttribute(attribute);
			if (att == null) {
				taskSpec.getDocument().getRootElement()
						.addAttribute(new Attribute(attribute, value));
			} else {
				att.setValue(value);
			}
		}

		update(taskSpec, taskState, taskOrigin);

	}

	@Override
	public String getActiveMemoryName() {
		return memoryName;
	}

	@Override
	public void resetActiveMemory(ActiveMemory activeMemory) {
		synchronized (memoryLock) {
			this.activeMemory = activeMemory;
		}
	}

	@Override
	public String info() {
		return "XcfTaskPatternActuator: This is the only implementation of the "
				+ "Actuator. You will have to cast to XcfTaskPatternActuator in "
				+ "order to access the methods.";
	}

}
