package de.unibi.citec.clf.bonsai.xcf.configuration;

/*
 * #LICENSE%
 * BonSAI [Core|XCF]
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
 * 
 * http://opensource.cit-ec.de/projects/bonsai
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import de.unibi.citec.clf.bonsai.core.CoreObjectFactory.CoreObjectCreationException;
import de.unibi.citec.clf.bonsai.core.CoreObjectFactory.SlotToConfigure;
import de.unibi.citec.clf.bonsai.core.CoreObjectFactory.WorkingMemoryToConfigure;
import de.unibi.citec.clf.bonsai.core.WorkingMemory;
import de.unibi.citec.clf.bonsai.xcf.AmWorkingMemory;
import de.unibi.citec.clf.bonsai.xcf.AmWorkingMemorySlot;
import java.util.HashSet;
import java.util.Set;
import org.apache.log4j.Logger;

/**
 * A Working Memory implementation using the xcf Active Memory.
 *
 * @author lziegler
 */
public class ConfiguredXcfAmWorkingMemory extends ConfiguredXcfWorkingMemory {

    private static Logger logger = Logger
            .getLogger(ConfiguredXcfAmWorkingMemory.class);
    private Set<ConfiguredXcfAmWorkingMemorySlot> slots = new HashSet<ConfiguredXcfAmWorkingMemorySlot>();

    /**
     * Check if memory to configure can be created from this instance.
     *
     * @param memoryToConfigure requested configuration
     * @return <code>true</code> if this configuration object is suitable for
     * the requested memory
     */
    public static boolean isSuitableFor(
            WorkingMemoryToConfigure memoryToConfigure) {

        boolean suitable = memoryToConfigure.getMemoryClass().equals(
                AmWorkingMemory.class);

        Set<SlotToConfigure> slots = memoryToConfigure.getSlots();
        for (SlotToConfigure slot : slots) {
            if (!AmWorkingMemorySlot.class
                    .isAssignableFrom(slot.getSlotClass())) {
                logger.warn("The configured slot \"" + slot + "\" can not "
                        + "be used by AmWorkingMemory because it does not "
                        + "implement the AmWorkingMemorySlot Interface");
                suitable = false;
            }
        }

        return suitable;
    }

    /**
     * Create a new specific working memory instance.
     *
     * @return new instance
     * @throws CoreObjectCreationException problem creating the new instance
     */
    @Override
    public WorkingMemory createInstance() throws CoreObjectCreationException {
        logger.debug("Create AmWorkingMemory instance with " + slots.size() + " memory slots.");
        return new AmWorkingMemory(getMemoryName(), slots);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String[] getActiveMemoriesToInitialize() {
        return new String[]{getMemoryName()};
    }

    @Override
    public void configureMemorySlots(Set<SlotToConfigure> slots) {
        logger.debug("Configure " + slots.size() + " memory slots.");
        for (SlotToConfigure slot : slots) {
            logger.debug("Adding slot: " + slot);
            if (AmWorkingMemorySlot.class.isAssignableFrom(slot.getSlotClass())) {
                this.slots.add(new ConfiguredXcfAmWorkingMemorySlot(slot
                        .getDataTypeClass(), slot.getSlotClass()));
            } else {
                logger.error("Trying to configure slot \"" + slot + "\", "
                        + "which can not be used by AmWorkingMemory because "
                        + "it does not implement the AmWorkingMemorySlot "
                        + "Interface");
            }
        }
    }
}
