package de.unibi.citec.clf.bonsai.xcf.configuration;

/*
 * #LICENSE%
 * BonSAI [Core|XCF]
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
 * 
 * http://opensource.cit-ec.de/projects/bonsai
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.util.Map;

import de.unibi.citec.clf.bonsai.core.Actuator;
import de.unibi.citec.clf.bonsai.core.CoreObjectFactory.ActuatorToConfigure;
import de.unibi.citec.clf.bonsai.core.CoreObjectFactory.CoreObjectCreationException;
import de.unibi.citec.clf.bonsai.xcf.actuators.XcfBtlMemoryActuator;

/**
 * Configuration object for {@link XcfBtlMemoryActuator}.
 * <p>
 * Available options:
 * <ul>
 * <li>{@link XcfBtlMemoryActuator#memoryName} (required): name of the
 * active memory to use</li>
 * </ul>
 *
 * @author lkettenb
 */
public class ConfiguredXcfBtlMemoryActuator extends ConfiguredXcfActuator {
    
    /**
     * Checks if the actuator that can be produced is suitable for the requested
     * configuration.
     * 
     * @param actuatorToConfigure
     *            requested configuration
     * @return <code>true</code> if suitable
     */
    /**
     * Option for remote server based classes defining the remote server name.
     */
	
	
    public static final String OPTION_XPATH = "xPath";
    
	private String xpath = "";

	public static boolean isSuitableFor(
            ActuatorToConfigure actuatorToConfigure) {
        return actuatorToConfigure.getActuatorClass().equals(
                XcfBtlMemoryActuator.class);
    }
    
    @Override
    public Actuator createInstance() throws CoreObjectCreationException {
        return new XcfBtlMemoryActuator(getServerName(), xpath);
    }
    
    @Override
    public void parseOptions(Map<String, String> options)
            throws IllegalArgumentException {

    	super.parseOptions(options);
    	
        if (options.containsKey(OPTION_XPATH)) {
            this.xpath = options.get(OPTION_XPATH);
        }
    }    
    
}
