package de.unibi.citec.clf.bonsai.xcf.actuators;

/*
 * #LICENSE%
 * BonSAI [Core|XCF]
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
 * 
 * http://opensource.cit-ec.de/projects/bonsai
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.assertEquals;

import java.io.IOException;

import net.sf.xcf.ActiveMemory;
import net.sf.xcf.event.MemoryEvent;
import net.sf.xcf.jam.ActiveMemoryRAM;
import net.sf.xcf.jam.EventDispatcher;
import net.sf.xcf.memory.MemoryException;
import net.sf.xcf.transport.XOPData;
import net.sf.xcf.util.SynchronizedQueue;
import net.sf.xcf.xml.XPath;
import nu.xom.Document;
import nu.xom.Element;
import nu.xom.Nodes;

import org.apache.log4j.BasicConfigurator;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.unibi.airobots.btl.data.geometry.Point2D;
import de.unibi.airobots.btl.units.LengthUnit;

public class XcfBtlMemoryActuatorTest {

	private static final String MEMORY_NAME = "testam";
	private ActiveMemory memory;

	/**
	 * Static initializer for logging.
	 */
	@BeforeClass
	public static void setUpLogging() {
		BasicConfigurator.configure();
	}

	/**
	 * Set up fixture.
	 * 
	 * @throws Exception
	 *             error...
	 */
	@Before
	public void setUp() throws Exception {
		EventDispatcher dummyDispatcher = new EventDispatcher() {

			public void fireEvent(MemoryEvent event) {
				// do nothing
			}
		};
		memory = new ActiveMemoryRAM(MEMORY_NAME, dummyDispatcher);
	}

	@Test
	public void test() throws IOException, MemoryException {

		// create actuator
		XcfBtlMemoryActuator<Point2D> actuator = new XcfBtlMemoryActuator<Point2D>(
				MEMORY_NAME, "/testpath/secondlevel");
		actuator.resetActiveMemory(memory);

		// create data
		Point2D point0 = new Point2D(1, 2, LengthUnit.METER);

		// insert at xpath
		actuator.insert(point0, Point2D.class);

		// check if data present
		SynchronizedQueue<XOPData> queue0 = memory.query(new XPath("/"));

		verifyElement(point0, queue0,
				"/testpath/secondlevel/" + Point2D.getBaseTag());

		SynchronizedQueue<XOPData> queue1 = memory
				.query(new XPath("/testpath"));
		
		verifyElement(point0, queue1,
				"/testpath/secondlevel/" + Point2D.getBaseTag());

		SynchronizedQueue<XOPData> queue2 = memory.query(new XPath("testpath"));
		
		verifyElement(point0, queue2,
				"/testpath/secondlevel/" + Point2D.getBaseTag());

		SynchronizedQueue<XOPData> queue3 = memory
				.query(new XPath("/falsepath"));

		assertEquals(0, queue3.getSize());
	}

	private void verifyElement(Point2D point, SynchronizedQueue<XOPData> queue,
			String xpath) {

		assertEquals(1, queue.getSize());
		Document doc = queue.next().getDocument();
		Nodes nodes0 = doc.query(xpath);
		assertEquals(1, nodes0.size());
		Element elem = (Element) nodes0.get(0);
		double x = Double.parseDouble(elem.getAttribute("x").getValue());
		double y = Double.parseDouble(elem.getAttribute("y").getValue());
		assertEquals(point.getX(LengthUnit.MILLIMETER), x, 0.0001);
		assertEquals(point.getY(LengthUnit.MILLIMETER), y, 0.0001);
	}

}
