package de.unibi.citec.clf.bonsai.xcf.configuration;

/*
 * #LICENSE%
 * BonSAI [Core|XCF]
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
 * 
 * http://opensource.cit-ec.de/projects/bonsai
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;

import org.apache.log4j.BasicConfigurator;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.unibi.airobots.btl.List;
import de.unibi.airobots.btl.data.vision1d.LaserData;
import de.unibi.airobots.btl.data.vision2d.ImageData;
import de.unibi.citec.clf.bonsai.TestUtils;
import de.unibi.citec.clf.bonsai.core.BonsaiManager;
import de.unibi.citec.clf.bonsai.core.BonsaiManager.ConfigurationException;
import de.unibi.citec.clf.bonsai.core.BonsaiManagerWrapper;
import de.unibi.citec.clf.bonsai.core.Sensor;
import de.unibi.citec.clf.bonsai.core.configuration.XmlConfigurationParser;

public class ConfiguredXcfSensorListableTest {

    /**
     * Setup logging.
     *
     * @throws Exception error
     */
    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
        BasicConfigurator.configure();
    }

    /**
     * Setup method constructing a new "singleton" instance of
     * {@link BonsaiManager}.
     */
    @Before
    public void setUp() {
        BonsaiManagerWrapper.wrapTestConstructNewInstance();
    }

    @Test
    public void test() throws IllegalStateException, ConfigurationException,
            URISyntaxException, IOException, InterruptedException {

        BonsaiManager.getInstance().configure(
                new URI(TestUtils.makeTestFileName("actualsensor.xml")),
                new XmlConfigurationParser());

        Class<List<LaserData>> listType = List.getListClass(LaserData.class);

        assertTrue(BonsaiManager.getInstance().canCreateSensor("LaserSensor",
                listType, LaserData.class));

        Sensor<List<LaserData>> laserSensor = BonsaiManager.getInstance().createSensor("LaserSensor", listType, LaserData.class);

        List<LaserData> data = laserSensor.readLast(100);

        assertNotNull(laserSensor);
        assertNull(data);
    }

    @Test(expected = IllegalArgumentException.class)
    public void testNotListable() throws IllegalStateException, ConfigurationException,
            URISyntaxException, IOException, InterruptedException {

        BonsaiManager.getInstance().configure(
                new URI(TestUtils.makeTestFileName("actualsensor.xml")),
                new XmlConfigurationParser());

        Class<List<ImageData>> listType = List.getListClass(ImageData.class);

        assertTrue(BonsaiManager.getInstance().canCreateSensor("LaserSensor",
                listType, LaserData.class));

        @SuppressWarnings("unused")
        Sensor<List<ImageData>> camSensor = BonsaiManager.getInstance().createSensor("CameraSensor", listType, ImageData.class);
    }
}
