package de.unibi.citec.clf.bonsai.xcf.sensors;

/*
 * #LICENSE%
 * BonSAI [Core|XCF]
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
 * 
 * http://opensource.cit-ec.de/projects/bonsai
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.*;

import java.io.IOException;

import net.sf.xcf.ActiveMemory;
import net.sf.xcf.event.MemoryEvent;
import net.sf.xcf.jam.ActiveMemoryRAM;
import net.sf.xcf.jam.EventDispatcher;
import net.sf.xcf.memory.MemoryException;
import net.sf.xcf.transport.XOPData;
import nu.xom.Document;

import org.apache.log4j.BasicConfigurator;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.unibi.citec.clf.bonsai.xcf.util.XPathUtilities;
import de.unibi.airobots.btl.data.geometry.Point2D;
import de.unibi.airobots.btl.units.LengthUnit;

public class XcfBtlMemorySensorTest {

	private static final String MEMORY_NAME = "testam";
	private ActiveMemory memory;

	/**
	 * Static initializer for logging.
	 */
	@BeforeClass
	public static void setUpLogging() {
		BasicConfigurator.configure();
	}

	/**
	 * Set up fixture.
	 * 
	 * @throws Exception
	 *             error...
	 */
	@Before
	public void setUp() throws Exception {
		EventDispatcher dummyDispatcher = new EventDispatcher() {

			public void fireEvent(MemoryEvent event) {
				// do nothing
			}
		};
		memory = new ActiveMemoryRAM(MEMORY_NAME, dummyDispatcher);
	}

	@Test
	public void testExisting() throws IOException, MemoryException, InterruptedException {

		// create sensor
		XcfBtlMemorySensor<Point2D> sensor = new XcfBtlMemorySensor<Point2D>(
				Point2D.class, MEMORY_NAME, "/testpath/secondlevel");
		sensor.resetActiveMemory(memory);

		// insert data into memory manually
		Point2D point0 = new Point2D(1, 2, LengthUnit.METER);
		Document doc0 = point0.toDocument(Point2D.class);
		doc0 = XPathUtilities.addParentXPath(doc0, "/testpath/secondlevel");
		
		memory.insert(new XOPData(doc0));
		
		// use sensor to read from memory
		Point2D point1 = sensor.readLast();
		
		assertEquals(point0, point1);
	}
	
	@Test
	public void testNoneExisting() throws IOException, MemoryException, InterruptedException {

		XcfBtlMemorySensor<Point2D> sensor = new XcfBtlMemorySensor<Point2D>(
				Point2D.class, MEMORY_NAME, "/testpath/secondlevel");
		sensor.resetActiveMemory(memory);

		Point2D point0 = new Point2D(1, 2, LengthUnit.METER);
		Document doc0 = point0.toDocument(Point2D.class);
		doc0 = XPathUtilities.addParentXPath(doc0, "/falsepath/secondlevel");
		
		memory.insert(new XOPData(doc0));
		
		Point2D point1 = sensor.readLast(200);
		
		assertNull(point1);
		
		doc0 = point0.toDocument(Point2D.class);
		
		memory.insert(new XOPData(doc0));
		
		point1 = sensor.readLast(200);
		
		assertNull(point1);
	}
}
